option(SPL_ENABLE_ASAN "Enable Address Sanitizer" false)
option(SPL_ENABLE_USAN "Enable Undefined-Behavior Sanitizer" false)
option(SPL_ENABLE_LSAN "Enable Leak Sanitizer" false)
option(SPL_ENABLE_MSAN "Enable Memory Sanitizer" false)

macro(spl_enable_sanitizer name)
	if ("${name}" MATCHES "asan")
		set(SANITIZER_COMPILE_FLAGS "-fsanitize=address")
		set(SANITIZER_LINKER_FLAGS "-fsanitize=address")
	elseif ("${name}" MATCHES "usan")
		set(SANITIZER_COMPILE_FLAGS "-fsanitize=undefined")
		set(SANITIZER_LINKER_FLAGS "-fsanitize=undefined")
	elseif ("${name}" MATCHES "leak")
		set(SANITIZER_COMPILE_FLAGS "-fsanitize=leak")
		set(SANITIZER_LINKER_FLAGS "-fsanitize=leak")
	elseif ("${name}" MATCHES "memory")
		set(SANITIZER_COMPILE_FLAGS "-fsanitize=memory")
		set(SANITIZER_LINKER_FLAGS "-fsanitize=memory")
	endif()
	set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${SANITIZER_COMPILE_FLAGS}")
	set(CMAKE_LD_FLAGS "${CMAKE_LD_FLAGS} ${SANITIZER_LINKER_FLAGS}")
endmacro()

message("SPL_ENABLE_ASAN ${SPL_ENABLE_ASAN}")
message("SPL_ENABLE_USAN ${SPL_ENABLE_USAN}")
message("SPL_ENABLE_LSAN ${SPL_ENABLE_LSAN}")
message("SPL_ENABLE_MSAN ${SPL_ENABLE_MSAN}")
if (SPL_ENABLE_ASAN)
	message("Enabling Address Sanitizer")
	spl_enable_sanitizer("asan")
endif()
if (SPL_ENABLE_USAN)
	message("Enabling Undefined-Behavior Sanitizer")
	spl_enable_sanitizer("usan")
endif()
if (SPL_ENABLE_LSAN)
	message("Enabling Leak Sanitizer")
	spl_enable_sanitizer("lsan")
endif()
if (SPL_ENABLE_MSAN)
	message("Enabling Memory Sanitizer")
	spl_enable_sanitizer("msan")
endif()

